<?
/**
 * Service
 * Базовый сервис
 * @package ru.vbinc.gm.framework.amfphp
 * @author GreyMag <greymag@gmail.com>
 * @copyright 2009
 * @version 0.7
 * @access public
 */
class Service
{
	/**
	 * Cоединение с БД
	 * @var DBConnect
	 */
	protected $_db;
	
	/**
	 * Успешный результат вызова метода или нет 
	 * @var bool
	 */
	protected $_success = true;
	
	/**
	 * Ассоциативный массив <strong>поле</strong> => <strong>значение</strong>, 
	 * который будет передан в качестве ответа
	 * @var array
	 */
	protected $_answer = array();
	
	/**
	 * Использовать поддержку нескольких языков или нет
	 * @var boolean
	 */
	protected $_useMultiLang = false;
	
	/**
	 * Номер ошибки
	 * @var int
	 */
	public $errno = 0;
	/**
	 * Текст ошибки
	 * @var string
	 */
	public $error = '';
	/**
	 * Данные которые отправляются в случае ошибки
	 * @var array
	 */
	protected $_errData = array();

	function __construct( $createDBC = true )
	{
		if( $createDBC ) $this->_db = Globals::getDBConnect();
	}

	/**
	 * Обработка исключения
	 * @param Exception $e
	 * @return array ассоциативный массив ответа на запрос
	 */
	protected function exception( Exception $e )
	{
		return $this->error( $e->getMessage(), $e->getCode() );
	}

	/**
	 * Ошибка
	 * @param string $message
	 * @param int $errno
	 * @param int $useLang использовать перевод или нет. По умолчанию - используется при условии включении мультиязычности
     * @return array ассоциативный массив ответа на запрос
	 */
	protected function error( $message = '', $errno = 0, $useLang = true )
	{
		if ($this->_useMultiLang && $useLang) {
			$message = $this->__( $message );
		}
		
		if (!empty( $message )) 
		  $this->error = $message;
		//elseif( !$this->error || $this->error == '' ) $this->error = $this->_db->error;
		// в дебаг режиме добавляем ошибку запроса к БД, если такая была
		if (Globals::isDebugMode()) {
			if ($this->_db->errno > 0) $this->error .= ' (DB error #' . $this->_db->errno . ': ' . $this->_db->error . ')';
		}

		if( $errno != 0 ) $this->errno = $errno;

		$this->_success = false;
		
		// а вдруг злобные хацкеры
        sleep( 1 );		

		return $this->answer();
	}
    
    /**
     * Возвращает строку в нужной локали
     * @param string $str
     */
    protected function __( $str )
    {
        return ($this->_useMultiLang) ? Lang::getString( $str ) : $str; 
    }

	/**
	 * Добавляет значение в ответ
	 * @param string $name название поля
	 * @param mixed $value добавляемое значение
	 */
	protected function add2Answer( $name, $value )
	{
		$this->_answer[$name] = $value;
	}
	
	/**
	 * Устанавливает данные, которые будет отправлены на клиент в случае ошибки
	 * @param array $data ассоциативный массив
	 */
	protected function setErrorData( $data ) {
		$this->_errData = $data;
	}
	
	/**
	 * Добавляет значение в данные при ошибке
	 * @param string $name название поля
	 * @param mixed $value добавляемое значение
	 */
	protected function add2ErrorData( $name, $value )
	{
		$this->_errData[$name] = $value;
	}

	/**
	 * Формирует ответ
     * @return array ассоциативный массив ответа на запрос
	 */
	protected function answer()
	{
		if (!$this->_success)
		{
			$this->_answer = array();
			if ($this->_errData != null && is_array( $this->_errData )) {
				foreach ($this->_errData as $key => $value) {
					$this->add2Answer( $key, $value );
				}
			} 
			$this->add2Answer( 'error', $this->error );
			$this->add2Answer( 'errno', $this->errno );
		}

		$this->add2Answer( 'success', $this->_success );

		return $this->_answer;
	}
}
?>