<?
/**
 * Настройки из БД
 * Настройки должны храниться в таблице следующей структуры:
 * <code>
 * CREATE TABLE `prefix_tablename` (
 * `option` varchar(20) NOT NULL COMMENT 'опция',
 * `value` text NOT NULL COMMENT 'её значение',
 * PRIMARY KEY  (`option`)
 * ) ENGINE=MyISAM DEFAULT CHARSET=utf8 COMMENT='таблица настроек';
 * </code>
 * Для более удобного использования предполагается, что в наследниках будет определён метод
 * <code>
 * public static function getInstance()
 * {
 *     return Options::getInstance( __CLASS__ );  
 * }
 * </code>
 * И основная работа с опциями будет вестись через него
 * @package ru.vbinc.gm.framework.utils
 * @author GreyMag
 * @version 0.3
 */
class Options
{
	/**
	 * Имя таблицы опций (без префикса).
	 * Выставляется обыяно при инициализации. 
	 * Префикс должен подставляться автоматически при использовании метода 
	 * <code>DBConnect::queryt()</code>
	 * @var String
	 */
	public static $tableName = '';
	
	/**
	 * 
	 * @var Options
	 */
	protected static $_instance;

	/**
	 * Получается одну или несколько опций из БД
	 * @param $optionName,... Неограниченное количество имён опций 
	 * @return false|string|array 
	 * Если не было передано ни одного имени опции - вернётся пустой массив.
	 * Если не было выбрано ни одной опции - вернется false.	 
	 * Если было передано одно имя опции, то вернется её значение.
	 * Если несколько - ассоциативный массив значений.
	 */
	public static function getOption( $optionName = '' )
	{
		if( func_num_args() == 0 || self::$tableName == ''
		      || !$dbConnect = Globals::getDBConnect() ) return false;

		$options = array();
		 
		$where = '';		
		$props = func_get_args();
		
		if (count( $props ) > 0) {
			foreach ($props as $optionName) 
			  $where .= ( ( $where == '' ) ? '' : ',' ) . "'" . $optionName . "'";
	
			$sql = "SELECT `option`, `value` FROM `%s` WHERE `option` IN (" . $where . ")";
			 
			if (!$query = $dbConnect->queryt( $sql, self::$tableName )) return false;
			while ($option = $query->fetch_assoc()) {
				$options[$option['option']] = $option['value'];
			}
			$query->close();
			
	        switch (count( $options ))
	        {
	            case 0 : return false;
	            case 1 : return array_pop( $options );
	            default : return $options;
	        }
		} else return $options;
	}

	/**
	 *
	 * @return Options
	 */
	protected static function getOptInstance( $class )
	{
		if( !self::$_instance ) {		
			if (!is_subclass_of( $class, __CLASS__)) 
			     throw new Exception( 'Класс для инстанции должен быть наследником Options' );	
			self::$_instance = new $class();
			self::$_instance->getOptions();
		}
		 
		return self::$_instance;
	}
	
	/**
	 * Преобразовательтипов
	 * @var TypeConverter
	 */
	protected $_typeConverter;
	
	function __construct()
	{		
		$this->_typeConverter = new TypeConverter( $this );
	}
	
	/**
	 * Загружает опции
	 */
	protected function getOptions()
	{
		// GMF2DO чтобы защищённые и приватные методы не грузились
		$options = call_user_func_array( array( 'Options', 'getOption' ),  
		                                 array_keys( get_object_vars( $this ) ) );
		
		if (!$options) throw new Exception( 'Ошибка при загрузке опций' );
		if (!is_array( $options )) $options = array( $options );
		 
		foreach ($options as $name => $value) {
			if (property_exists( $this, $name )) {
				$this->$name = $this->_typeConverter->setType( $name, $value );
			}
		}

		$this->initOptions();
	}
	
	/**
	 * Проводит дополнительные действия с опциями после загрузки, если требуется
	 */
	protected function initOptions()
	{
		
	}
}
?>